@@ -1,24 +1,28 @@
 import os
 import sys
 import platform
+import traceback
+from fnmatch import fnmatch
 
 from cStringIO import StringIO
 
+from PyQt4 import QtCore
+from PyQt4.QtCore import SIGNAL
+
 import cola
+from cola import compat
 from cola import i18n
 from cola import core
 from cola import errors
 from cola import gitcfg
 from cola import gitcmds
 from cola import utils
-from cola import signals
-from cola import cmdfactory
 from cola import difftool
 from cola.diffparse import DiffParser
+from cola.interaction import Interaction
 from cola.models import selection
 
 _notifier = cola.notifier()
-_factory = cmdfactory.factory()
 _config = gitcfg.instance()
 
 
@@ -35,6 +39,15 @@ class BaseCommand(object):
         """Return this command's name."""
         return self.__class__.__name__
 
+    def prepare(self):
+        """Prepare to run the command.
+
+        This is performed in a separate thread before do()
+        is invoked.
+
+        """
+        pass
+
     def do(self):
         raise NotImplementedError('%s.do() is unimplemented' % self.name())
 
@@ -62,10 +75,10 @@ class Command(BaseCommand):
 
     def do(self):
         """Perform the operation."""
-        self.model.set_diff_text(self.new_diff_text)
         self.model.set_filename(self.new_filename)
         self.model.set_head(self.new_head)
         self.model.set_mode(self.new_mode)
+        self.model.set_diff_text(self.new_diff_text)
 
     def undo(self):
         """Undo the operation."""
@@ -77,6 +90,8 @@ class Command(BaseCommand):
 
 class AmendMode(Command):
     """Try to amend a commit."""
+    LAST_MESSAGE = None
+
     def __init__(self, amend):
         Command.__init__(self)
         self.undoable = True
@@ -88,6 +103,7 @@ class AmendMode(Command):
             self.new_mode = self.model.mode_amend
             self.new_head = 'HEAD^'
             self.new_commitmsg = self.model.prev_commitmsg()
+            AmendMode.LAST_MESSAGE = self.model.commitmsg
             return
         # else, amend unchecked, regular commit
         self.new_mode = self.model.mode_none
@@ -97,18 +113,9 @@ class AmendMode(Command):
         # If we're going back into new-commit-mode then search the
         # undo stack for a previous amend-commit-mode and grab the
         # commit message at that point in time.
-        if not _factory.undostack:
-            return
-        undo_count = len(_factory.undostack)
-        for i in xrange(undo_count):
-            # Find the latest AmendMode command
-            idx = undo_count - i - 1
-            cmdobj = _factory.undostack[idx]
-            if type(cmdobj) is not AmendMode:
-                continue
-            if cmdobj.amending:
-                self.new_commitmsg = cmdobj.old_commitmsg
-                break
+        if AmendMode.LAST_MESSAGE is not None:
+            self.new_commitmsg = AmendMode.LAST_MESSAGE
+            AmendMode.LAST_MESSAGE = None
 
     def do(self):
         """Leave/enter amend mode."""
@@ -116,14 +123,14 @@ class AmendMode(Command):
         if self.amending:
             if os.path.exists(self.model.git.git_path('MERGE_HEAD')):
                 self.skip = True
-                _notifier.broadcast(signals.amend, False)
-                _factory.prompt_user(signals.information,
-                                    'Oops! Unmerged',
-                                    'You are in the middle of a merge.\n'
-                                    'You cannot amend while merging.')
+                _notifier.broadcast(_notifier.AMEND, False)
+                Interaction.information(
+                        'Oops! Unmerged',
+                        'You are in the middle of a merge.\n'
+                        'You cannot amend while merging.')
                 return
         self.skip = False
-        _notifier.broadcast(signals.amend, self.amending)
+        _notifier.broadcast(_notifier.AMEND, self.amending)
         self.model.set_commitmsg(self.new_commitmsg)
         Command.do(self)
         self.model.update_file_status()
@@ -156,7 +163,7 @@ class ApplyDiffSelection(Command):
                                       self.offset,
                                       self.selection,
                                       apply_to_worktree=self.apply_to_worktree)
-        _notifier.broadcast(signals.log_cmd, status, output)
+        Interaction.log_status(status, output, '')
         # Redo the diff to show changes
         if self.staged:
             diffcmd = DiffStaged([self.model.filename])
@@ -182,7 +189,7 @@ class ApplyPatches(Command):
                                                with_status=True,
                                                with_stderr=True)
             # Log the git-am command
-            _notifier.broadcast(signals.log_cmd, status, output)
+            Interaction.log_status(status, output, '')
 
             if num_patches > 1:
                 diff = self.model.git.diff('HEAD^!', stat=True)
@@ -197,23 +204,34 @@ class ApplyPatches(Command):
 
         self.model.update_file_status()
 
-        _factory.prompt_user(signals.information,
-                            'Patch(es) Applied',
-                            '%d patch(es) applied:\n\n%s' %
-                            (len(self.patches),
-                             '\n'.join(map(os.path.basename, self.patches))))
+        Interaction.information(
+                'Patch(es) Applied',
+                '%d patch(es) applied:\n\n%s' %
+                (len(self.patches),
+                 '\n'.join(map(os.path.basename, self.patches))))
 
 
-class HeadChangeCommand(Command):
-    """Changes the model's current head."""
-    def __init__(self, treeish):
-        Command.__init__(self)
-        self.new_head = treeish
-        self.new_diff_text = ''
+class Archive(BaseCommand):
+    def __init__(self, ref, fmt, prefix, filename):
+        BaseCommand.__init__(self)
+        self.ref = ref
+        self.fmt = fmt
+        self.prefix = prefix
+        self.filename = filename
 
     def do(self):
-        Command.do(self)
-        self.model.update_file_status()
+        fp = open(core.encode(self.filename), 'wb')
+        cmd = ['git', 'archive', '--format='+self.fmt]
+        if self.fmt in ('tgz', 'tar.gz'):
+            cmd.append('-9')
+        if self.prefix:
+            cmd.append('--prefix=' + self.prefix)
+        cmd.append(self.ref)
+        proc = utils.start_command(cmd, stdout=fp)
+        out, err = proc.communicate()
+        fp.close()
+        status = proc.returncode
+        Interaction.log_status(status, out or '', err or '')
 
 
 class Checkout(Command):
@@ -232,7 +250,7 @@ class Checkout(Command):
     def do(self):
         status, output = self.model.git.checkout(with_stderr=True,
                                                  with_status=True, *self.argv)
-        _notifier.broadcast(signals.log_cmd, status, output)
+        Interaction.log_status(status, output, '')
         if self.checkout_branch:
             self.model.update_status()
         else:
@@ -241,8 +259,9 @@ class Checkout(Command):
 
 class CheckoutBranch(Checkout):
     """Checkout a branch."""
-    def __init__(self, branch, checkout_branch=True):
-        Checkout.__init__(self, [branch])
+    def __init__(self, branch):
+        args = [branch]
+        Checkout.__init__(self, args, checkout_branch=True)
 
 
 class CherryPick(Command):
@@ -279,14 +298,15 @@ class Commit(ResetMode):
         self.new_commitmsg = ''
 
     def do(self):
-        status, output = self.model.commit_with_msg(self.msg, amend=self.amend)
+        tmpfile = utils.tmp_filename('commit-message')
+        status, output = self.model.commit_with_msg(self.msg, tmpfile, amend=self.amend)
         if status == 0:
             ResetMode.do(self)
             self.model.set_commitmsg(self.new_commitmsg)
             title = 'Commit: '
         else:
             title = 'Commit failed: '
-        _notifier.broadcast(signals.log_cmd, status, title+output)
+        Interaction.log_status(status, title+output, '')
 
 
 class Ignore(Command):
@@ -305,9 +325,8 @@ class Ignore(Command):
                 current_list = utils.slurp('.gitignore')
                 new_additions = new_additions + current_list
             utils.write('.gitignore', new_additions)
-            _notifier.broadcast(signals.log_cmd,
-                                0,
-                                'Added to .gitignore:\n%s' % for_status)
+            Interaction.log_status(
+                    0, 'Added to .gitignore:\n%s' % for_status, '')
             self.model.update_file_status()
 
 
@@ -326,12 +345,12 @@ class Delete(Command):
                     os.remove(filename)
                     rescan=True
                 except:
-                    _factory.prompt_user(signals.information,
-                                        'Error'
-                                        'Deleting "%s" failed.' % filename)
+                    Interaction.information(
+                            'Error', 'Deleting "%s" failed' % filename)
         if rescan:
             self.model.update_file_status()
 
+
 class DeleteBranch(Command):
     """Delete a git branch."""
     def __init__(self, branch):
@@ -345,7 +364,7 @@ class DeleteBranch(Command):
             output = 'E' + output[1:]
         else:
             title = 'Info: '
-        _notifier.broadcast(signals.log_cmd, status, title + output)
+        Interaction.log_status(status, title + output)
 
 
 class Diff(Command):
@@ -357,32 +376,14 @@ class Diff(Command):
             return
         opts = {}
         if cached:
-            cached = not self.model.read_only()
-            opts = dict(ref=self.model.head)
-
+            opts['ref'] = self.model.head
         self.new_filename = filenames[0]
         self.old_filename = self.model.filename
-        if not self.model.read_only():
-            if self.model.mode != self.model.mode_amend:
-                self.new_mode = self.model.mode_worktree
+        self.new_mode = self.model.mode_worktree
         self.new_diff_text = gitcmds.diff_helper(filename=self.new_filename,
                                                  cached=cached, **opts)
 
 
-class DiffMode(HeadChangeCommand):
-    """Enter diff mode and clear the model's diff text."""
-    def __init__(self, treeish):
-        HeadChangeCommand.__init__(self, treeish)
-        self.new_mode = self.model.mode_diff
-
-
-class DiffExprMode(HeadChangeCommand):
-    """Enter diff-expr mode and clear the model's diff text."""
-    def __init__(self, treeish):
-        HeadChangeCommand.__init__(self, treeish)
-        self.new_mode = self.model.mode_diff_expr
-
-
 class Diffstat(Command):
     """Perform a diffstat and set the model's diff text."""
     def __init__(self):
@@ -393,33 +394,26 @@ class Diffstat(Command):
                                    M=True,
                                    stat=True)
         self.new_diff_text = core.decode(diff)
-        if not self.model.read_only():
-            if self.model.mode != self.model.mode_amend:
-                self.new_mode = self.model.mode_worktree
+        self.new_mode = self.model.mode_worktree
 
 
 class DiffStaged(Diff):
     """Perform a staged diff on a file."""
     def __init__(self, filenames):
         Diff.__init__(self, filenames, cached=True)
-        if not self.model.read_only():
-            if self.model.mode != self.model.mode_amend:
-                self.new_mode = self.model.mode_index
+        self.new_mode = self.model.mode_index
 
 
 class DiffStagedSummary(Command):
     def __init__(self):
         Command.__init__(self)
-        cached = not self.model.read_only()
         diff = self.model.git.diff(self.model.head,
-                                   cached=cached,
+                                   cached=True,
                                    no_color=True,
                                    patch_with_stat=True,
                                    M=True)
         self.new_diff_text = core.decode(diff)
-        if not self.model.read_only():
-            if self.model.mode != self.model.mode_amend:
-                self.new_mode = self.model.mode_index
+        self.new_mode = self.model.mode_index
 
 
 class Difftool(Command):
@@ -430,34 +424,47 @@ class Difftool(Command):
         self.filenames = filenames
 
     def do(self):
-        if not self.filenames:
-            return
-        args = []
-        if self.staged and not self.model.read_only():
-            args.append('--cached')
-        if self.model.head != 'HEAD':
-            args.append(self.model.head)
-        args.append('--')
-        args.extend(self.filenames)
-        difftool.launch(args)
+        difftool.launch_with_head(self.filenames,
+                                  self.staged, self.model.head)
 
 
 class Edit(Command):
     """Edit a file using the configured gui.editor."""
+    NAME = 'Edit'
+    SHORTCUT = 'Ctrl+E'
+
     def __init__(self, filenames, line_number=None):
         Command.__init__(self)
         self.filenames = filenames
         self.line_number = line_number
 
     def do(self):
+        if not self.filenames:
+            return
         filename = self.filenames[0]
         if not os.path.exists(filename):
             return
         editor = self.model.editor()
-        if editor == 'gvim' and self.line_number:
-            utils.fork([editor, filename, '+'+self.line_number])
+        opts = []
+
+        if self.line_number is None:
+            opts = self.filenames
         else:
-            utils.fork([editor, filename])
+            # Single-file w/ line-numbers (likely from grep)
+            editor_opts = {
+                    '*vim*': ['+'+self.line_number, filename],
+                    '*emacs*': ['+'+self.line_number, filename],
+                    '*textpad*': ['%s(%s,0)' % (filename, self.line_number)],
+                    '*notepad++*': ['-n'+self.line_number, filename],
+            }
+
+            opts = self.filenames
+            for pattern, opt in editor_opts.items():
+                if fnmatch(editor, pattern):
+                    opts = opt
+                    break
+
+        utils.fork(utils.shell_split(editor) + opts)
 
 
 class FormatPatch(Command):
@@ -469,15 +476,28 @@ class FormatPatch(Command):
 
     def do(self):
         status, output = gitcmds.format_patchsets(self.to_export, self.revs)
-        _notifier.broadcast(signals.log_cmd, status, output)
+        Interaction.log_status(status, output, '')
 
 
-class GrepMode(Command):
-    def __init__(self, txt):
-        """Perform a git-grep."""
-        Command.__init__(self)
-        self.new_mode = self.model.mode_grep
-        self.new_diff_text = core.decode(self.model.git.grep(txt, n=True))
+class LaunchDifftool(BaseCommand):
+    NAME = 'Launch Diff Tool'
+    SHORTCUT = 'Ctrl+D'
+
+    def __init__(self):
+        BaseCommand.__init__(self)
+
+    def do(self):
+        difftool.run()
+
+
+class LaunchEditor(Edit):
+    NAME = 'Launch Editor'
+    SHORTCUT = 'Ctrl+E'
+
+    def __init__(self):
+        s = cola.selection()
+        allfiles = s.staged + s.unmerged + s.modified + s.untracked
+        Edit.__init__(self, allfiles)
 
 
 class LoadCommitMessage(Command):
@@ -532,6 +552,30 @@ class LoadPreviousMessage(Command):
         self.model.set_commitmsg(self.old_commitmsg)
 
 
+class Merge(Command):
+    def __init__(self, revision, no_commit, squash):
+        Command.__init__(self)
+        self.revision = revision
+        self.no_commit = no_commit
+        self.squash = squash
+
+    def do(self):
+        squash = self.squash
+        revision = self.revision
+        no_commit = self.no_commit
+        msg = gitcmds.merge_message(revision)
+
+        status, output = self.model.git.merge('-m', msg,
+                                              revision,
+                                              no_commit=no_commit,
+                                              squash=squash,
+                                              with_stderr=True,
+                                              with_status=True)
+
+        Interaction.log_status(status, output, '')
+        self.model.update_status()
+
+
 class Mergetool(Command):
     """Launch git-mergetool on a list of paths."""
     def __init__(self, paths):
@@ -541,14 +585,53 @@ class Mergetool(Command):
     def do(self):
         if not self.paths:
             return
-        utils.fork(['git', 'mergetool', '--no-prompt', '--'] + self.paths)
+        if utils.is_win32():
+            utils.fork(['git', 'mergetool', '--no-prompt', '--'] + self.paths)
+        else:
+            utils.fork(['xterm', '-e',
+                        'git', 'mergetool', '--no-prompt', '--'] + self.paths)
+
+
+class OpenDefaultApp(BaseCommand):
+    """Open a file using the OS default."""
+    NAME = 'Open Using Default Application'
+    SHORTCUT = 'Space'
+
+    def __init__(self, filenames):
+        BaseCommand.__init__(self)
+        if utils.is_darwin():
+            launcher = 'open'
+        else:
+            launcher = 'xdg-open'
+        self.launcher = launcher
+        self.filenames = filenames
+
+    def do(self):
+        if not self.filenames:
+            return
+        utils.fork([self.launcher] + self.filenames)
+
+
+class OpenParentDir(OpenDefaultApp):
+    """Open parent directories using the OS default."""
+    NAME = 'Open Parent Directory'
+    SHORTCUT = 'Shift+Space'
+
+    def __init__(self, filenames):
+        OpenDefaultApp.__init__(self, filenames)
+
+    def do(self):
+        if not self.filenames:
+            return
+        dirs = set(map(os.path.dirname, self.filenames))
+        utils.fork([self.launcher] + dirs)
 
 
 class OpenRepo(Command):
     """Launches git-cola on a repo."""
     def __init__(self, dirname):
         Command.__init__(self)
-        self.new_directory = dirname
+        self.new_directory = utils.quote_repopath(dirname)
 
     def do(self):
         self.model.set_directory(self.new_directory)
@@ -560,7 +643,7 @@ class Clone(Command):
     def __init__(self, url, destdir, spawn=True):
         Command.__init__(self)
         self.url = url
-        self.new_directory = destdir
+        self.new_directory = utils.quote_repopath(destdir)
         self.spawn = spawn
 
     def do(self):
@@ -578,27 +661,15 @@ class Rescan(Command):
         self.model.update_status()
 
 
-rescan_and_refresh = 'rescan_and_refresh'
-
 class RescanAndRefresh(Command):
     """Rescans for changes."""
+    NAME = 'Rescan'
+    SHORTCUT = 'Ctrl+R'
+
     def do(self):
         self.model.update_status(update_index=True)
 
 
-class ReviewBranchMode(Command):
-    """Enter into review-branch mode."""
-    def __init__(self, branch):
-        Command.__init__(self)
-        self.new_mode = self.model.mode_review
-        self.new_head = gitcmds.merge_base_parent(branch)
-        self.new_diff_text = ''
-
-    def do(self):
-        Command.do(self)
-        self.model.update_status()
-
-
 class RunConfigAction(Command):
     """Run a user-configured action, typically from the "Tools" menu"""
     def __init__(self, name):
@@ -609,7 +680,7 @@ class RunConfigAction(Command):
     def do(self):
         for env in ('FILENAME', 'REVISION', 'ARGS'):
             try:
-                del os.environ[env]
+                compat.unsetenv(env)
             except KeyError:
                 pass
         rev = None
@@ -626,48 +697,47 @@ class RunConfigAction(Command):
         if opts.get('needsfile'):
             filename = selection.filename()
             if not filename:
-                _factory.prompt_user(signals.information,
-                                     'Please select a file',
-                                     '"%s" requires a selected file' % cmd)
-                return
-            os.environ['FILENAME'] = filename
+                Interaction.information(
+                        'Please select a file',
+                        '"%s" requires a selected file' % cmd)
+                return False
+            compat.putenv('FILENAME', filename)
 
         if opts.get('revprompt') or opts.get('argprompt'):
             while True:
-                ok = _factory.prompt_user(signals.run_config_action, cmd, opts)
+                ok = Interaction.confirm_config_action(cmd, opts)
                 if not ok:
-                    return
+                    return False
                 rev = opts.get('revision')
                 args = opts.get('args')
                 if opts.get('revprompt') and not rev:
                     title = 'Invalid Revision'
                     msg = 'The revision expression cannot be empty.'
-                    _factory.prompt_user(signals.critical, title, msg)
+                    Interaction.critical(title, msg)
                     continue
                 break
 
         elif opts.get('confirm'):
             title = os.path.expandvars(opts.get('title'))
             prompt = os.path.expandvars(opts.get('prompt'))
-            if not _factory.prompt_user(signals.question, title, prompt):
+            if Interaction.question(title, prompt):
                 return
         if rev:
-            os.environ['REVISION'] = rev
+            compat.putenv('REVISION', rev)
         if args:
-            os.environ['ARGS'] = args
+            compat.putenv('ARGS', args)
         title = os.path.expandvars(cmd)
-        _notifier.broadcast(signals.log_cmd, 0, 'running: ' + title)
+        Interaction.log('running: ' + title)
         cmd = ['sh', '-c', cmd]
 
         if opts.get('noconsole'):
-            status, out, err = utils.run_command(cmd, flag_error=False)
+            status, out, err = utils.run_command(cmd)
         else:
-            status, out, err = _factory.prompt_user(signals.run_command,
-                                                    title, cmd)
+            status, out, err = Interaction.run_command(title, cmd)
 
-        _notifier.broadcast(signals.log_cmd, status,
-                            'stdout: %s\nstatus: %s\nstderr: %s' %
-                                (out.rstrip(), status, err.rstrip()))
+        Interaction.log_status(status,
+                               out and 'stdout: %s' % out,
+                               err and 'stderr: %s' % err)
 
         if not opts.get('norescan'):
             self.model.update_status()
@@ -683,18 +753,32 @@ class SetDiffText(Command):
 
 class ShowUntracked(Command):
     """Show an untracked file."""
-    # We don't actually do anything other than set the mode right now.
-    # TODO check the mimetype for the file and handle things
-    # generically.
     def __init__(self, filenames):
         Command.__init__(self)
-        if not self.model.read_only():
-            if self.model.mode != self.model.mode_amend:
-                self.new_mode = self.model.mode_untracked
-        # TODO new_diff_text = utils.file_preview(filenames[0])
+        self.filenames = filenames
+        self.new_mode = self.model.mode_untracked
+        self.new_diff_text = ''
+
+    def prepare(self):
+        filenames = self.filenames
+        self.new_diff_text = self.diff_text_for(filenames[0])
+
+    def diff_text_for(self, filename):
+        size = _config.get('cola.readsize', 1024 * 2)
+        try:
+            result = utils.slurp(filename, size=size)
+        except:
+            result = ''
+
+        if len(result) == size:
+            result += '...'
+        return result
 
 
 class SignOff(Command):
+    NAME = 'Sign Off'
+    SHORTCUT = 'Ctrl+I'
+
     def __init__(self):
         Command.__init__(self)
         self.undoable = True
@@ -723,18 +807,24 @@ class SignOff(Command):
 
 class Stage(Command):
     """Stage a set of paths."""
+    NAME = 'Stage'
+    SHORTCUT = 'Ctrl+S'
+
     def __init__(self, paths):
         Command.__init__(self)
         self.paths = paths
 
     def do(self):
         msg = 'Staging: %s' % (', '.join(self.paths))
-        _notifier.broadcast(signals.log_cmd, 0, msg)
+        Interaction.log(msg)
         self.model.stage_paths(self.paths)
 
 
 class StageModified(Stage):
     """Stage all modified files."""
+    NAME = 'Stage Modified'
+    SHORTCUT = 'Ctrl+S'
+
     def __init__(self):
         Stage.__init__(self, None)
         self.paths = self.model.modified
@@ -742,6 +832,9 @@ class StageModified(Stage):
 
 class StageUnmerged(Stage):
     """Stage all modified files."""
+    NAME = 'Stage Unmerged'
+    SHORTCUT = 'Ctrl+S'
+
     def __init__(self):
         Stage.__init__(self, None)
         self.paths = self.model.unmerged
@@ -749,6 +842,9 @@ class StageUnmerged(Stage):
 
 class StageUntracked(Stage):
     """Stage all untracked files."""
+    NAME = 'Stage Untracked'
+    SHORTCUT = 'Ctrl+S'
+
     def __init__(self):
         Stage.__init__(self, None)
         self.paths = self.model.untracked
@@ -767,7 +863,7 @@ class Tag(Command):
         log_msg = 'Tagging: "%s" as "%s"' % (self._revision, self._name)
         opts = {}
         if self._message:
-            opts['F'] = self.model.tmp_filename()
+            opts['F'] = utils.tmp_filename('tag-message')
             utils.write(opts['F'], self._message)
 
         if self._sign:
@@ -791,20 +887,23 @@ class Tag(Command):
         if output:
             log_msg += '\nOutput:\n%s' % output
 
-        _notifier.broadcast(signals.log_cmd, status, log_msg)
+        Interaction.log_status(status, log_msg, '')
         if status == 0:
             self.model.update_status()
 
 
 class Unstage(Command):
     """Unstage a set of paths."""
+    NAME = 'Unstage'
+    SHORTCUT = 'Ctrl+S'
+
     def __init__(self, paths):
         Command.__init__(self)
         self.paths = paths
 
     def do(self):
         msg = 'Unstaging: %s' % (', '.join(self.paths))
-        _notifier.broadcast(signals.log_cmd, 0, msg)
+        Interaction.log(msg)
         self.model.unstage_paths(self.paths)
 
 
@@ -820,6 +919,19 @@ class UnstageSelected(Unstage):
         Unstage.__init__(self, cola.selection_model().staged)
 
 
+class Untrack(Command):
+    """Unstage a set of paths."""
+    def __init__(self, paths):
+        Command.__init__(self)
+        self.paths = paths
+
+    def do(self):
+        msg = 'Untracking: %s' % (', '.join(self.paths))
+        Interaction.log(msg)
+        status, out = self.model.untrack_paths(self.paths)
+        Interaction.log_status(status, out, '')
+
+
 class UntrackedSummary(Command):
     """List possible .gitignore rules as the diff text."""
     def __init__(self):
@@ -831,12 +943,9 @@ class UntrackedSummary(Command):
         if untracked:
             io.write('# possible .gitignore rule%s:\n' % suffix)
             for u in untracked:
-                io.write('/'+core.encode(u))
+                io.write('/'+core.encode(u)+'\n')
         self.new_diff_text = core.decode(io.getvalue())
-
-        if not self.model.read_only():
-            if self.model.mode != self.model.mode_amend:
-                self.new_mode = self.model.mode_untracked
+        self.new_mode = self.model.mode_untracked
 
 
 class UpdateFileStatus(Command):
@@ -848,33 +957,31 @@ class UpdateFileStatus(Command):
 class VisualizeAll(Command):
     """Visualize all branches."""
     def do(self):
-        browser = self.model.history_browser()
-        utils.fork([browser, '--all'])
+        browser = utils.shell_split(self.model.history_browser())
+        utils.fork(browser + ['--all'])
 
 
 class VisualizeCurrent(Command):
     """Visualize all branches."""
     def do(self):
-        browser = self.model.history_browser()
-        utils.fork([browser, self.model.currentbranch])
+        browser = utils.shell_split(self.model.history_browser())
+        utils.fork(browser + [self.model.currentbranch])
 
 
 class VisualizePaths(Command):
     """Path-limited visualization."""
     def __init__(self, paths):
         Command.__init__(self)
-        browser = self.model.history_browser()
+        browser = utils.shell_split(self.model.history_browser())
         if paths:
-            self.argv = [browser] + paths
+            self.argv = browser + paths
         else:
-            self.argv = [browser]
+            self.argv = browser
 
     def do(self):
         utils.fork(self.argv)
 
 
-visualize_revision = 'visualize_revision'
-
 class VisualizeRevision(Command):
     """Visualize a specific revision."""
     def __init__(self, revision, paths=None):
@@ -883,7 +990,7 @@ class VisualizeRevision(Command):
         self.paths = paths
 
     def do(self):
-        argv = [self.model.history_browser()]
+        argv = utils.shell_split(self.model.history_browser())
         if self.revision:
             argv.append(self.revision)
         if self.paths:
@@ -892,65 +999,98 @@ class VisualizeRevision(Command):
         utils.fork(argv)
 
 
-def register():
+def run(cls, *args, **opts):
     """
-    Register signal mappings with the factory.
+    Returns a callback that runs a command
 
-    These commands are automatically created and run when
-    their corresponding signal is broadcast by the notifier.
+    If the caller of run() provides args or opts then those are
+    used instead of the ones provided by the invoker of the callback.
 
     """
-    signal_to_command_map = {
-        signals.amend_mode: AmendMode,
-        signals.apply_diff_selection: ApplyDiffSelection,
-        signals.apply_patches: ApplyPatches,
-        signals.clone: Clone,
-        signals.checkout: Checkout,
-        signals.checkout_branch: CheckoutBranch,
-        signals.cherry_pick: CherryPick,
-        signals.commit: Commit,
-        signals.delete: Delete,
-        signals.delete_branch: DeleteBranch,
-        signals.diff: Diff,
-        signals.diff_mode: DiffMode,
-        signals.diff_expr_mode: DiffExprMode,
-        signals.diff_staged: DiffStaged,
-        signals.diffstat: Diffstat,
-        signals.difftool: Difftool,
-        signals.edit: Edit,
-        signals.format_patch: FormatPatch,
-        signals.grep: GrepMode,
-        signals.ignore: Ignore,
-        signals.load_commit_message: LoadCommitMessage,
-        signals.load_commit_template: LoadCommitTemplate,
-        signals.load_previous_message: LoadPreviousMessage,
-        signals.modified_summary: Diffstat,
-        signals.mergetool: Mergetool,
-        signals.open_repo: OpenRepo,
-        signals.rescan: Rescan,
-        signals.rescan_and_refresh: RescanAndRefresh,
-        signals.reset_mode: ResetMode,
-        signals.review_branch_mode: ReviewBranchMode,
-        signals.run_config_action: RunConfigAction,
-        signals.set_diff_text: SetDiffText,
-        signals.show_untracked: ShowUntracked,
-        signals.signoff: SignOff,
-        signals.stage: Stage,
-        signals.stage_modified: StageModified,
-        signals.stage_unmerged: StageUnmerged,
-        signals.stage_untracked: StageUntracked,
-        signals.staged_summary: DiffStagedSummary,
-        signals.tag: Tag,
-        signals.unstage: Unstage,
-        signals.unstage_all: UnstageAll,
-        signals.unstage_selected: UnstageSelected,
-        signals.untracked_summary: UntrackedSummary,
-        signals.update_file_status: UpdateFileStatus,
-        signals.visualize_all: VisualizeAll,
-        signals.visualize_current: VisualizeCurrent,
-        signals.visualize_paths: VisualizePaths,
-        signals.visualize_revision: VisualizeRevision,
-    }
+    def runner(*local_args, **local_opts):
+        if args or opts:
+            do(cls, *args, **opts)
+        else:
+            do(cls, *local_args, **local_opts)
 
-    for signal, cmd in signal_to_command_map.iteritems():
-        _factory.add_global_command(signal, cmd)
+    return runner
+
+
+def do(cls, *args, **opts):
+    """Run a command in-place"""
+    do_cmd(cls(*args, **opts))
+
+
+def do_cmd(cmd):
+    try:
+        cmd.do()
+    except StandardError, e:
+        exc_type, exc_value, exc_tb = sys.exc_info()
+        details = traceback.format_exception(exc_type, exc_value, exc_tb)
+        details = '\n'.join(details)
+        msg = _exception_message(e)
+        Interaction.critical('Oops', message=msg, details=details)
+
+
+def bg(parent, cls, *args, **opts):
+    """
+    Returns a callback that runs a command
+
+    If the caller of run() provides args or opts then those are
+    used instead of the ones provided by the invoker of the callback.
+
+    """
+    def runner(*local_args, **local_opts):
+        if args or opts:
+            background(parent, cls, *args, **opts)
+        else:
+            background(parent, cls, *local_args, **local_opts)
+
+    return runner
+
+
+# Holds a reference to background tasks to avoid PyQt4 segfaults
+ALL_TASKS = set()
+
+
+def background(parent, cls, *args, **opts):
+    cmd = cls(*args, **opts)
+    task = AsyncCommand(parent, cmd)
+    ALL_TASKS.add(task)
+    QtCore.QThreadPool.globalInstance().start(task)
+
+
+
+class RunCommand(QtCore.QObject):
+    def __init__(self, cmd, task, parent):
+        QtCore.QObject.__init__(self, parent)
+        self.cmd = cmd
+        self.task = task
+        self.connect(self, SIGNAL('command_ready'), self.do)
+
+    def run(self):
+        self.cmd.prepare()
+        self.emit(SIGNAL('command_ready'))
+
+    def do(self):
+        do_cmd(self.cmd)
+        try:
+            ALL_TASKS.remove(self.task)
+        except:
+            pass
+
+
+class AsyncCommand(QtCore.QRunnable):
+    def __init__(self, parent, cmd):
+        QtCore.QRunnable.__init__(self)
+        self.runner = RunCommand(cmd, self, parent)
+
+    def run(self):
+        self.runner.run()
+
+
+def _exception_message(e):
+    if hasattr(e, 'msg'):
+        return e.msg
+    else:
+        return str(e)
