@@ -1,11 +1,41 @@
 from PyQt4 import QtGui
 from PyQt4 import QtCore
+from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
+import cola
 from cola import utils
 from cola import qtutils
 from cola import gitcmds
 from cola.widgets import defs
+from cola.widgets import standard
+
+
+def run():
+    s = cola.selection()
+    if s.staged:
+        selection = s.staged
+    elif s.unmerged:
+        selection = s.unmerged
+    elif s.modified:
+        selection = s.modified
+    elif s.untracked:
+        selection = s.untracked
+    else:
+        return
+    model = cola.model()
+    launch_with_head(selection, bool(s.staged), model.head)
+
+
+def launch_with_head(filenames, staged, head):
+    args = []
+    if staged:
+        args.append('--cached')
+    if head != 'HEAD':
+        args.append(head)
+    args.append('--')
+    args.extend(filenames)
+    launch(args)
 
 
 def launch(args):
@@ -16,25 +46,33 @@ def launch(args):
 
 
 def diff_commits(parent, a, b):
-    dlg = FileDiffDialog(parent, a, b)
+    dlg = FileDiffDialog(parent, a=a, b=b)
+    dlg.show()
+    dlg.raise_()
+    return dlg.exec_() == QtGui.QDialog.Accepted
+
+
+def diff_expression(parent, expr):
+    dlg = FileDiffDialog(parent, expr=expr)
     dlg.show()
     dlg.raise_()
     return dlg.exec_() == QtGui.QDialog.Accepted
 
 
 class FileDiffDialog(QtGui.QDialog):
-    def __init__(self, parent, a, b):
+    def __init__(self, parent, a=None, b=None, expr=None):
         QtGui.QDialog.__init__(self, parent)
+        self.setAttribute(Qt.WA_MacMetalStyle)
         self.a = a
         self.b = b
+        self.expr = expr
 
         self.setWindowTitle('Select File(s)')
-        self._tree = QtGui.QTreeWidget(self)
-        self._tree.setAlternatingRowColors(True)
+        self.setWindowModality(QtCore.Qt.WindowModal)
+
+        self._tree = standard.TreeWidget(self)
         self._tree.setRootIsDecorated(False)
         self._tree.setSelectionMode(self._tree.ExtendedSelection)
-        self._tree.setUniformRowHeights(True)
-        self._tree.setAllColumnsShowFocus(True)
         self._tree.setHeaderHidden(True)
 
         self._diff_btn = QtGui.QPushButton('Compare')
@@ -57,8 +95,6 @@ class FileDiffDialog(QtGui.QDialog):
         self._layt.addLayout(self._button_layt)
         self.setLayout(self._layt)
 
-        qtutils.add_close_action(self)
-
         self.connect(self._tree, SIGNAL('itemSelectionChanged()'),
                      self._tree_selection_changed)
 
@@ -66,16 +102,28 @@ class FileDiffDialog(QtGui.QDialog):
                      SIGNAL('itemDoubleClicked(QTreeWidgetItem*,int)'),
                      self._tree_double_clicked)
 
-        self.connect(self._diff_btn, SIGNAL('clicked()'), self.diff)
-        self.connect(self._close_btn, SIGNAL('clicked()'), self.close)
+        qtutils.connect_button(self._diff_btn, self.diff)
+        qtutils.connect_button(self._close_btn, self.close)
+        qtutils.add_close_action(self)
 
-        self.diff_arg = (self.a, self.b)
+        if self.expr:
+            self.diff_arg = tuple(utils.shell_split(self.expr))
+        elif self.b is None:
+            self.diff_arg = (self.a,)
+        else:
+            self.diff_arg = (self.a, self.b)
 
         self.resize(720, 420)
 
 
     def exec_(self):
-        filenames = gitcmds.diff_filenames(*self.diff_arg)
+        if self.expr:
+            filenames = gitcmds.diff_filenames(*self.diff_arg)
+        elif self.b is None:
+            filenames = gitcmds.diff_index_filenames(self.a)
+        else:
+            filenames = gitcmds.diff_filenames(*self.diff_arg)
+
         if not filenames:
             details = ('"git diff --name-only %s" returned an empty list' %
                        ' '.join(self.diff_arg))
