@@ -7,158 +7,149 @@ from PyQt4 import QtGui
 from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
-import cola
+from cola import cmds
 from cola import core
 from cola import gitcmds
 from cola import guicmds
 from cola import merge
-from cola import settings
-from cola import signals
 from cola import gitcfg
+from cola import prefs
 from cola import qtutils
 from cola import qtcompat
-from cola import qt
 from cola import resources
+from cola import settings
 from cola import stash
 from cola import utils
 from cola import version
 from cola.bookmarks import manage_bookmarks
 from cola.classic import cola_classic
 from cola.classic import classic_widget
-from cola.controllers import compare
-from cola.controllers import createtag
-from cola.controllers import search
-from cola.controllers.createbranch import create_new_branch
 from cola.dag import git_dag
 from cola.git import git
-from cola.prefs import diff_font
-from cola.prefs import PreferencesModel
-from cola.prefs import preferences
-from cola.qt import create_button
+from cola.interaction import Interaction
 from cola.qt import create_dock
 from cola.qt import create_menu
 from cola.qtutils import add_action
 from cola.qtutils import connect_action
 from cola.qtutils import connect_action_bool
-from cola.qtutils import connect_button
-from cola.qtutils import emit
-from cola.qtutils import log
-from cola.qtutils import relay_signal
 from cola.qtutils import tr
-from cola.views import standard
+from cola.widgets import action
 from cola.widgets import cfgactions
+from cola.widgets import editremotes
+from cola.widgets import remote
 from cola.widgets.about import launch_about_dialog
-from cola.widgets.archive import GitArchiveDialog
 from cola.widgets.about import show_shortcuts
+from cola.widgets.archive import GitArchiveDialog
 from cola.widgets.commitmsg import CommitMessageEditor
-from cola.widgets.diff import DiffTextEdit
+from cola.widgets.compare import compare_branches
+from cola.widgets.createtag import create_tag
+from cola.widgets.createbranch import create_new_branch
+from cola.widgets.diff import DiffEditor
+from cola.widgets.log import LogWidget
 from cola.widgets.recent import browse_recent
 from cola.widgets.status import StatusWidget
+from cola.widgets.search import search
+from cola.widgets.standard import MainWindow
 
 
-class MainView(standard.MainWindow):
+class MainView(MainWindow):
     def __init__(self, model, parent):
-        standard.MainWindow.__init__(self, parent)
+        MainWindow.__init__(self, parent)
         # Default size; this is thrown out when save/restore is used
         self.resize(987, 610)
         self.model = model
-        self.prefs_model = prefs_model = PreferencesModel()
+        self.prefs_model = prefs_model = prefs.PreferencesModel()
 
         # Internal field used by import/export_state().
         # Change this whenever dockwidgets are removed.
-        self.widget_version = 1
+        self.widget_version = 2
 
         # Keeps track of merge messages we've seen
         self.merge_message_hash = ''
 
         self.setAcceptDrops(True)
+        self.setAttribute(Qt.WA_MacMetalStyle)
 
         # Dockwidget options
         qtcompat.set_common_dock_options(self)
 
-        self.classic_dockable = gitcfg.instance().get('cola.classicdockable')
-
+        cfg = gitcfg.instance()
+        self.classic_dockable = (cfg.get('cola.browserdockable') or
+                                 cfg.get('cola.classicdockable'))
         if self.classic_dockable:
-            self.classicdockwidget = create_dock('Cola Classic', self)
+            self.classicdockwidget = create_dock('Browser', self)
             self.classicwidget = classic_widget(self)
             self.classicdockwidget.setWidget(self.classicwidget)
 
         # "Actions" widget
-        self.actionsdockwidget = create_dock('Actions', self)
-        self.actionsdockwidgetcontents = qt.QFlowLayoutWidget(self)
-        layout = self.actionsdockwidgetcontents.layout()
-        self.stage_button = create_button(text='Stage', layout=layout)
-        self.unstage_button = create_button(text='Unstage', layout=layout)
-        self.rescan_button = create_button(text='Rescan', layout=layout)
-        self.fetch_button = create_button(text='Fetch...', layout=layout)
-        self.push_button = create_button(text='Push...', layout=layout)
-        self.pull_button = create_button(text='Pull...', layout=layout)
-        self.stash_button = create_button(text='Stash...', layout=layout)
-        self.alt_button = create_button(text='Exit Diff Mode', layout=layout)
-        self.alt_button.hide()
-        layout.addStretch()
+        self.actionsdockwidget = create_dock('Action', self)
+        self.actionsdockwidgetcontents = action.ActionButtons(self)
         self.actionsdockwidget.setWidget(self.actionsdockwidgetcontents)
+        self.actionsdockwidget.toggleViewAction().setChecked(False)
+        self.actionsdockwidget.hide()
 
         # "Repository Status" widget
-        self.statusdockwidget = create_dock('Repository Status', self)
-        self.statusdockwidget.setWidget(StatusWidget(self))
+        self.statuswidget = StatusWidget(self)
+        self.statusdockwidget = create_dock('Status', self)
+        self.statusdockwidget.setWidget(self.statuswidget)
 
         # "Commit Message Editor" widget
         self.position_label = QtGui.QLabel()
         font = qtutils.default_monospace_font()
         font.setPointSize(int(font.pointSize() * 0.8))
         self.position_label.setFont(font)
-        self.commitdockwidget = create_dock('Commit Message Editor', self)
+        self.commitdockwidget = create_dock('Commit', self)
         titlebar = self.commitdockwidget.titleBarWidget()
         titlebar.add_corner_widget(self.position_label)
 
         self.commitmsgeditor = CommitMessageEditor(model, self)
-        relay_signal(self, self.commitmsgeditor, SIGNAL(signals.amend_mode))
-        relay_signal(self, self.commitmsgeditor, SIGNAL(signals.signoff))
-        relay_signal(self, self.commitmsgeditor,
-                     SIGNAL(signals.load_previous_message))
         self.commitdockwidget.setWidget(self.commitmsgeditor)
 
-        # "Command Output" widget
-        logwidget = qtutils.logger()
-        logwidget.setFont(diff_font())
-        self.logdockwidget = create_dock('Command Output', self)
-        self.logdockwidget.setWidget(logwidget)
+        # "Console" widget
+        self.logwidget = LogWidget()
+        self.logdockwidget = create_dock('Console', self)
+        self.logdockwidget.setWidget(self.logwidget)
+        self.logdockwidget.toggleViewAction().setChecked(False)
+        self.logdockwidget.hide()
 
         # "Diff Viewer" widget
-        self.diffdockwidget = create_dock('Diff Viewer', self)
-        self.diff_viewer = DiffTextEdit(self.diffdockwidget)
-        self.diffdockwidget.setWidget(self.diff_viewer)
+        self.diffdockwidget = create_dock('Diff', self)
+        self.diffeditor = DiffEditor(self.diffdockwidget)
+        self.diffdockwidget.setWidget(self.diffeditor)
 
         # All Actions
         self.menu_unstage_all = add_action(self,
-                'Unstage All', emit(self, signals.unstage_all))
+                'Unstage All', cmds.run(cmds.UnstageAll))
         self.menu_unstage_all.setIcon(qtutils.icon('remove.svg'))
 
         self.menu_unstage_selected = add_action(self,
-                'Unstage From Commit', emit(self, signals.unstage_selected))
+                'Unstage From Commit', cmds.run(cmds.UnstageSelected))
         self.menu_unstage_selected.setIcon(qtutils.icon('remove.svg'))
 
         self.menu_show_diffstat = add_action(self,
-                'Diffstat', emit(self, signals.diffstat), 'Alt+D')
+                'Diffstat', cmds.run(cmds.Diffstat), 'Alt+D')
 
         self.menu_stage_modified = add_action(self,
                 'Stage Changed Files To Commit',
-                emit(self, signals.stage_modified), 'Alt+A')
+                cmds.run(cmds.StageModified), 'Alt+A')
         self.menu_stage_modified.setIcon(qtutils.icon('add.svg'))
 
         self.menu_stage_untracked = add_action(self,
-                'Stage All Untracked', emit(self, signals.stage_untracked), 'Alt+U')
+                'Stage All Untracked', cmds.run(cmds.StageUntracked), 'Alt+U')
         self.menu_stage_untracked.setIcon(qtutils.icon('add.svg'))
 
         self.menu_export_patches = add_action(self,
                 'Export Patches...', guicmds.export_patches, 'Alt+E')
         self.menu_preferences = add_action(self,
-                'Preferences', lambda: preferences(model=prefs_model),
+                'Preferences', self.preferences,
                 QtGui.QKeySequence.Preferences, 'Ctrl+O')
 
+        self.menu_edit_remotes = add_action(self,
+                'Edit Remotes...', lambda: editremotes.edit().exec_())
         self.menu_rescan = add_action(self,
-                'Rescan', emit(self, signals.rescan_and_refresh), 'Ctrl+R')
+                cmds.RescanAndRefresh.NAME,
+                cmds.run(cmds.RescanAndRefresh),
+                cmds.RescanAndRefresh.SHORTCUT)
         self.menu_rescan.setIcon(qtutils.reload_icon())
 
         self.menu_browse_recent = add_action(self,
@@ -178,7 +169,7 @@ class MainView(standard.MainWindow):
         self.menu_manage_bookmarks = add_action(self,
                 'Bookmarks...', manage_bookmarks)
         self.menu_grep = add_action(self,
-                'Grep', guicmds.grep)
+                'Grep', guicmds.grep, 'Ctrl+G')
         self.menu_merge_local = add_action(self,
                 'Merge...', merge.local_merge)
 
@@ -186,11 +177,11 @@ class MainView(standard.MainWindow):
                 'Abort Merge...', merge.abort_merge)
 
         self.menu_fetch = add_action(self,
-                'Fetch...', guicmds.fetch)
+                'Fetch...', remote.fetch)
         self.menu_push = add_action(self,
-                'Push...', guicmds.push)
+                'Push...', remote.push)
         self.menu_pull = add_action(self,
-                'Pull...', guicmds.pull)
+                'Pull...', remote.pull)
 
         self.menu_open_repo = add_action(self,
                 'Open...', guicmds.open_repo)
@@ -198,8 +189,6 @@ class MainView(standard.MainWindow):
 
         self.menu_stash = add_action(self,
                 'Stash...', stash.stash, 'Alt+Shift+S')
-        self.menu_branch_compare = add_action(self,
-                'Branches...', compare.branch_compare)
 
         self.menu_clone_repo = add_action(self,
                 'Clone...', guicmds.clone_repo)
@@ -214,33 +203,33 @@ class MainView(standard.MainWindow):
                 show_shortcuts,
                 QtCore.Qt.Key_Question)
 
-        self.menu_commit_compare = add_action(self,
-                'Commits...', compare.compare)
-        self.menu_commit_compare_file = add_action(self,
-                'Commits Touching File...', compare.compare_file)
         self.menu_visualize_current = add_action(self,
                 'Visualize Current Branch...',
-                emit(self, signals.visualize_current))
+                cmds.run(cmds.VisualizeCurrent))
         self.menu_visualize_all = add_action(self,
                 'Visualize All Branches...',
-                emit(self, signals.visualize_all))
+                cmds.run(cmds.VisualizeAll))
         self.menu_search_commits = add_action(self,
-                'Search...', search.search)
+                'Search...', search)
         self.menu_browse_branch = add_action(self,
                 'Browse Current Branch...', guicmds.browse_current)
         self.menu_browse_other_branch = add_action(self,
                 'Browse Other Branch...', guicmds.browse_other)
         self.menu_load_commitmsg_template = add_action(self,
                 'Get Commit Message Template',
-                emit(self, signals.load_commit_template))
+                cmds.run(cmds.LoadCommitTemplate))
         self.menu_help_about = add_action(self,
                 'About', launch_about_dialog)
+
         self.menu_branch_diff = add_action(self,
-                'SHA-1...', guicmds.branch_diff)
+                'SHA-1...', guicmds.diff_revision)
         self.menu_diff_expression = add_action(self,
                 'Expression...', guicmds.diff_expression)
+        self.menu_branch_compare = add_action(self,
+                'Branches...', compare_branches)
+
         self.menu_create_tag = add_action(self,
-                'Create Tag...', createtag.create_tag)
+                'Create Tag...', create_tag)
 
         self.menu_create_branch = add_action(self,
                 'Create...', create_new_branch, 'Ctrl+B')
@@ -256,7 +245,7 @@ class MainView(standard.MainWindow):
                 'Review...', guicmds.review_branch)
 
         self.menu_classic = add_action(self,
-                'Cola Classic...', cola_classic)
+                'Browser...', cola_classic)
         self.menu_classic.setIcon(qtutils.git_icon())
 
         self.menu_dag = add_action(self,
@@ -264,11 +253,14 @@ class MainView(standard.MainWindow):
         self.menu_dag.setIcon(qtutils.git_icon())
 
         # Relayed actions
-        status_tree = self.statusdockwidget.widget().tree
-        self.addAction(status_tree.up)
-        self.addAction(status_tree.down)
-        self.addAction(status_tree.process_selection)
-        self.addAction(status_tree.launch_difftool)
+        if not self.classic_dockable:
+            # These shortcuts conflict with those from the
+            # 'Browser' widget so don't register them when
+            # the browser is a dockable tool.
+            status_tree = self.statusdockwidget.widget().tree
+            self.addAction(status_tree.up)
+            self.addAction(status_tree.down)
+            self.addAction(status_tree.process_selection)
 
         # Create the application menu
         self.menubar = QtGui.QMenuBar(self)
@@ -278,9 +270,12 @@ class MainView(standard.MainWindow):
         self.file_menu.addAction(self.menu_preferences)
         self.file_menu.addSeparator()
         self.file_menu.addAction(self.menu_open_repo)
+        self.menu_open_recent = self.file_menu.addMenu(tr('Open Recent'))
+        self.file_menu.addSeparator()
         self.file_menu.addAction(self.menu_clone_repo)
         self.file_menu.addAction(self.menu_manage_bookmarks)
         self.file_menu.addSeparator()
+        self.file_menu.addAction(self.menu_edit_remotes)
         self.file_menu.addAction(self.menu_rescan)
         self.file_menu.addSeparator()
         self.file_menu.addAction(self.menu_browse_recent)
@@ -346,10 +341,7 @@ class MainView(standard.MainWindow):
         self.diff_menu = create_menu('&Diff', self.menubar)
         self.diff_menu.addAction(self.menu_branch_diff)
         self.diff_menu.addAction(self.menu_diff_expression)
-        self.diff_menu.addSeparator()
         self.diff_menu.addAction(self.menu_branch_compare)
-        self.diff_menu.addAction(self.menu_commit_compare)
-        self.diff_menu.addAction(self.menu_commit_compare_file)
         self.diff_menu.addSeparator()
         self.diff_menu.addAction(self.menu_show_diffstat)
         # Add to menubar
@@ -378,48 +370,36 @@ class MainView(standard.MainWindow):
         self.setMenuBar(self.menubar)
 
         # Arrange dock widgets
-        top = Qt.TopDockWidgetArea
+        left = Qt.LeftDockWidgetArea
+        right = Qt.RightDockWidgetArea
         bottom = Qt.BottomDockWidgetArea
 
-        self.addDockWidget(top, self.commitdockwidget)
-        if self.classic_dockable:
-            self.addDockWidget(top, self.classicdockwidget)
-        self.addDockWidget(top, self.statusdockwidget)
-        self.addDockWidget(top, self.actionsdockwidget)
-        self.addDockWidget(bottom, self.logdockwidget)
+        self.addDockWidget(left, self.commitdockwidget)
         if self.classic_dockable:
+            self.addDockWidget(left, self.classicdockwidget)
             self.tabifyDockWidget(self.classicdockwidget, self.commitdockwidget)
-        self.tabifyDockWidget(self.logdockwidget, self.diffdockwidget)
+        self.addDockWidget(left, self.diffdockwidget)
+        self.addDockWidget(bottom, self.actionsdockwidget)
+        self.addDockWidget(bottom, self.logdockwidget)
+        self.tabifyDockWidget(self.actionsdockwidget, self.logdockwidget)
+
+        self.addDockWidget(right, self.statusdockwidget)
 
         # Listen for model notifications
-        model.add_message_observer(model.message_mode_changed,
-                                   self._mode_changed)
+        model.add_observer(model.message_updated, self._update_view)
 
-        model.add_message_observer(model.message_updated,
-                                   self._update_view)
-
-        prefs_model.add_message_observer(prefs_model.message_config_updated,
-                                         self._config_updated)
+        prefs_model.add_observer(prefs_model.message_config_updated,
+                                 self._config_updated)
 
         # Set a default value
         self.show_cursor_position(1, 0)
 
-        # Add button callbacks
-        connect_button(self.rescan_button,
-                       emit(self, signals.rescan_and_refresh))
-        connect_button(self.alt_button, emit(self, signals.reset_mode))
-        connect_button(self.fetch_button, guicmds.fetch)
-        connect_button(self.push_button, guicmds.push)
-        connect_button(self.pull_button, guicmds.pull)
-        connect_button(self.stash_button, stash.stash)
-
-        connect_button(self.stage_button, self.stage)
-        connect_button(self.unstage_button, self.unstage)
+        self.connect(self.menu_open_recent, SIGNAL('aboutToShow()'),
+                     self.build_recent_menu)
 
         self.connect(self.commitmsgeditor, SIGNAL('cursorPosition(int,int)'),
                      self.show_cursor_position)
         self.connect(self, SIGNAL('update'), self._update_callback)
-        self.connect(self, SIGNAL('apply_state'), self.apply_state)
         self.connect(self, SIGNAL('install_config_actions'),
                      self._install_config_actions)
 
@@ -427,45 +407,73 @@ class MainView(standard.MainWindow):
         self._config_task = None
         self.install_config_actions()
 
+        self.dockwidgets = (
+                self.logdockwidget,
+                self.commitdockwidget,
+                self.statusdockwidget,
+                self.diffdockwidget,
+                self.actionsdockwidget,
+        )
         # Restore saved settings
-        self._gui_state_task = None
-        self._load_gui_state()
+        if not qtutils.apply_state(self):
+            self.set_initial_size()
 
-        log(0, self.model.git_version + '\ncola version ' + version.version())
+        self.statusdockwidget.widget().setFocus()
+
+        # Route command output here
+        Interaction.log_status = self.logwidget.log_status
+        Interaction.log = self.logwidget.log
+
+        Interaction.log(version.git_version_str() +
+                        '\ncola version ' + version.version())
+
+    def set_initial_size(self):
+        self.statuswidget.set_initial_size()
+        self.commitmsgeditor.set_initial_size()
 
     # Qt overrides
     def closeEvent(self, event):
         """Save state in the settings manager."""
-        qtutils.save_state(self)
-        standard.MainWindow.closeEvent(self, event)
+        s = settings.Settings()
+        s.add_recent(core.decode(os.getcwd()))
+        qtutils.save_state(self, handler=s)
+        MainWindow.closeEvent(self, event)
+
+    def build_recent_menu(self):
+        recent = settings.Settings().recent
+        menu = self.menu_open_recent
+        menu.clear()
+        for r in recent:
+            name = os.path.basename(r)
+            directory = os.path.dirname(r)
+            text = u'%s %s %s' % (name, unichr(0x2192), directory)
+            menu.addAction(text, cmds.run(cmds.OpenRepo, r))
 
     # Accessors
     mode = property(lambda self: self.model.mode)
 
-    def _mode_changed(self, mode):
-        """React to mode changes; hide/show the "Exit Diff Mode" button."""
-        if mode in (self.model.mode_review,
-                    self.model.mode_diff,
-                    self.model.mode_diff_expr):
-            height = self.stage_button.minimumHeight()
-            self.alt_button.setMinimumHeight(height)
-            self.alt_button.show()
-        else:
-            self.alt_button.setMinimumHeight(1)
-            self.alt_button.hide()
-
     def _config_updated(self, source, config, value):
         if config == 'cola.fontdiff':
+            # The diff font
             font = QtGui.QFont()
             if not font.fromString(value):
                 return
-            qtutils.logger().setFont(font)
-            self.diff_viewer.setFont(font)
+            self.logwidget.setFont(font)
+            self.diffeditor.setFont(font)
             self.commitmsgeditor.setFont(font)
 
         elif config == 'cola.tabwidth':
             # variable-tab-width setting
-            self.diff_viewer.set_tab_width(value)
+            self.diffeditor.set_tabwidth(value)
+            self.commitmsgeditor.set_tabwidth(value)
+
+        elif config == 'cola.linebreak':
+            # enables automatic line breaks
+            self.commitmsgeditor.set_linebreak(value)
+
+        elif config == 'cola.textwidth':
+            # text width used for line wrapping
+            self.commitmsgeditor.set_textwidth(value)
 
     def install_config_actions(self):
         """Install .gitconfig-defined actions"""
@@ -492,7 +500,7 @@ class MainView(standard.MainWindow):
         menu = self.actions_menu
         menu.addSeparator()
         for name in names:
-            menu.addAction(name, emit(self, signals.run_config_action, name))
+            menu.addAction(name, cmds.run(cmds.RunConfigAction, name))
 
     def _update_view(self):
         self.emit(SIGNAL('update'))
@@ -505,17 +513,13 @@ class MainView(standard.MainWindow):
         self.commitdockwidget.setToolTip(msg)
 
         title = '%s: %s' % (self.model.project, branch)
-        if self.mode in (self.model.mode_diff, self.model.mode_diff_expr):
-            title += ' *** diff mode***'
-        elif self.mode == self.model.mode_review:
-            title += ' *** review mode***'
-        elif self.mode == self.model.mode_amend:
-            title += ' *** amending ***'
+        if self.mode == self.model.mode_amend:
+            title += ' ** amending **'
         self.setWindowTitle(title)
 
         self.commitmsgeditor.set_mode(self.mode)
 
-        if not self.model.read_only() and self.mode != self.model.mode_amend:
+        if not self.model.amending():
             # Check if there's a message file in .git/
             merge_msg_path = gitcmds.merge_message_path()
             if merge_msg_path is None:
@@ -524,96 +528,71 @@ class MainView(standard.MainWindow):
             if merge_msg_hash == self.merge_message_hash:
                 return
             self.merge_message_hash = merge_msg_hash
-            cola.notifier().broadcast(signals.load_commit_message,
-                                      core.decode(merge_msg_path))
+            cmds.do(cmds.LoadCommitMessage, core.decode(merge_msg_path))
 
     def apply_state(self, state):
         """Imports data for save/restore"""
         # 1 is the widget version; change when widgets are added/removed
-        standard.MainWindow.apply_state(self, state)
-        qtutils.apply_window_state(self, state, 1)
+        MainWindow.apply_state(self, state)
+        result = qtutils.apply_window_state(self, state, self.widget_version)
+        for widget in self.dockwidgets:
+            widget.titleBarWidget().update_tooltips()
+        return result
 
     def export_state(self):
         """Exports data for save/restore"""
-        state = standard.MainWindow.export_state(self)
+        state = MainWindow.export_state(self)
         return qtutils.export_window_state(self, state, self.widget_version)
 
-    def _load_gui_state(self):
-        """Restores the gui from the preferences file."""
-        self._gui_state_task = self._start_gui_state_loading_thread()
-
-    def _start_gui_state_loading_thread(self):
-        """Do expensive file reading and json decoding in the background"""
-        class LoadGUIStateTask(QtCore.QRunnable):
-            def __init__(self, sender):
-                QtCore.QRunnable.__init__(self)
-                self._sender = sender
-            def run(self):
-                state = settings.Settings().get_gui_state(self._sender)
-                self._sender.emit(SIGNAL('apply_state'), state)
-
-        task = LoadGUIStateTask(self)
-        QtCore.QThreadPool.globalInstance().start(task)
-        return task
-
     def setup_dockwidget_tools_menu(self):
         # Hotkeys for toggling the dock widgets
+        if utils.is_darwin():
+            optkey = 'Meta'
+        else:
+            optkey = 'Ctrl'
         dockwidgets = (
-            (self.logdockwidget, 'Alt+0'),
-            (self.commitdockwidget, 'Alt+1'),
-            (self.statusdockwidget, 'Alt+2'),
-            (self.diffdockwidget, 'Alt+3'),
-            (self.actionsdockwidget, 'Alt+4'),
+            (optkey + '+0', self.logdockwidget),
+            (optkey + '+1', self.commitdockwidget),
+            (optkey + '+2', self.statusdockwidget),
+            (optkey + '+3', self.diffdockwidget),
+            (optkey + '+4', self.actionsdockwidget),
         )
-        for dockwidget, shortcut in dockwidgets:
+        for shortcut, dockwidget in dockwidgets:
             # Associate the action with the shortcut
-            action = dockwidget.toggleViewAction()
-            action.setShortcut(shortcut)
-            self.tools_menu.addAction(action)
+            toggleview = dockwidget.toggleViewAction()
+            toggleview.setShortcut(shortcut)
+            self.tools_menu.addAction(toggleview)
             def showdock(show, dockwidget=dockwidget):
                 if show:
                     dockwidget.raise_()
-                    dockwidget.widget().setFocus(True)
+                    dockwidget.widget().setFocus()
                 else:
-                    self.setFocus(True)
-            self.addAction(action)
-            connect_action_bool(action, showdock)
+                    self.setFocus()
+            self.addAction(toggleview)
+            connect_action_bool(toggleview, showdock)
 
             # Create a new shortcut Shift+<shortcut> that gives focus
-            action = QtGui.QAction(self)
-            action.setShortcut('Shift+' + shortcut)
+            toggleview = QtGui.QAction(self)
+            toggleview.setShortcut('Shift+' + shortcut)
             def focusdock(dockwidget=dockwidget, showdock=showdock):
                 if dockwidget.toggleViewAction().isChecked():
                     showdock(True)
                 else:
                     dockwidget.toggleViewAction().trigger()
-            self.addAction(action)
-            connect_action(action, focusdock)
+            self.addAction(toggleview)
+            connect_action(toggleview, focusdock)
+
+    def preferences(self):
+        return prefs.preferences(model=self.prefs_model, parent=self)
 
     def save_archive(self):
         ref = git.rev_parse('HEAD')
         shortref = ref[:7]
         GitArchiveDialog.save(ref, shortref, self)
 
-    def stage(self):
-        """Stage selected files, or all files if no selection exists."""
-        paths = cola.selection_model().unstaged
-        if not paths:
-            cola.notifier().broadcast(signals.stage_modified)
-        else:
-            cola.notifier().broadcast(signals.stage, paths)
-
-    def unstage(self):
-        """Unstage selected files, or all files if no selection exists."""
-        paths = cola.selection_model().staged
-        if not paths:
-            cola.notifier().broadcast(signals.unstage_all)
-        else:
-            cola.notifier().broadcast(signals.unstage, paths)
-
     def dragEnterEvent(self, event):
         """Accepts drops"""
-        standard.MainWindow.dragEnterEvent(self, event)
+        MainWindow.dragEnterEvent(self, event)
         event.acceptProposedAction()
 
     def dropEvent(self, event):
@@ -628,8 +607,7 @@ class MainView(standard.MainWindow):
         dirs.sort()
         for d in dirs:
             patches.extend(self._gather_patches(d))
-        # Broadcast the patches to apply
-        cola.notifier().broadcast(signals.apply_patches, patches)
+        cmds.do(cmds.ApplyPatches, patches)
 
     def _gather_patches(self, path):
         """Find patches in a subdirectory"""
