@@ -10,7 +10,6 @@ import subprocess
 import sys
 import time
 
-from cola import git
 from cola import core
 from cola import resources
 from cola.compat import hashlib
@@ -54,17 +53,6 @@ def add_parents(path_entry_set):
     return path_entry_set
 
 
-def run_cmd(command):
-    """
-    Run arguments as a command and return output.
-
-    >>> run_cmd(["echo", "hello", "world"])
-    'hello world'
-
-    """
-    return git.Git.execute(command)
-
-
 def ident_file_type(filename):
     """Returns an icon based on the contents of filename."""
     if os.path.exists(filename):
@@ -229,10 +217,10 @@ def dirname(path):
     return path.rsplit('/', 1)[0]
 
 
-def slurp(path):
+def slurp(path, size=-1):
     """Slurps a filepath into a string."""
     fh = open(core.encode(path))
-    slushy = core.read(fh)
+    slushy = core.read(fh, size=size)
     fh.close()
     return core.decode(slushy)
 
@@ -269,7 +257,12 @@ def word_wrap(text, tabwidth, limit):
 
     """
     lines = []
+    # Acked-by:, Signed-off-by:, Helped-by:, etc.
+    special_tag_rgx = re.compile('^[a-zA-Z-]+:')
     for line in text.split('\n'):
+        if special_tag_rgx.match(line):
+            lines.append(line)
+            continue
         linelen = 0
         words = []
         for idx, word in enumerate(line.split(' ')):
@@ -468,6 +461,7 @@ class ProgressIndicator(object):
 
 
 def start_command(args, cwd=None, shell=False, add_env=None,
+                  universal_newlines=False,
                   stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                   stderr=subprocess.PIPE):
     """Start the given command, and return a subprocess object.
@@ -480,12 +474,12 @@ def start_command(args, cwd=None, shell=False, add_env=None,
         env = os.environ.copy()
         env.update(add_env)
     return subprocess.Popen(args, bufsize=1, stdin=stdin, stdout=stdout,
-                            stderr=stderr, cwd=cwd, shell=shell,
-                            env=env, universal_newlines=True)
+                            stderr=stderr, cwd=cwd, shell=shell, env=env,
+                            universal_newlines=universal_newlines)
 
 
 def run_command(args, cwd=None, shell=False, add_env=None,
-                flag_error=True):
+                flag_error=False):
     """Run the given command to completion, and return its results.
 
     This provides a simpler interface to the subprocess module.
