@@ -1,5 +1,5 @@
 from PyQt4 import QtGui
-from PyQt4 import QtCore
+from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
 import os
@@ -9,33 +9,25 @@ if __name__ == '__main__':
     sys.path.insert(1,
             os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
 
+from cola import cmds
 from cola import qtutils
 from cola.git import git
 from cola.qt import ExpandableGroupBox
 from cola.widgets import defs
-from cola.dag.model import archive
+
 
 
 class GitArchiveDialog(QtGui.QDialog):
-
     @staticmethod
-    def create(ref, shortref, parent):
+    def save(ref, shortref, parent=None):
         dlg = GitArchiveDialog(ref, shortref, parent)
         if dlg.exec_() != dlg.Accepted:
             return None
         return dlg
 
-    @classmethod
-    def save(cls, ref, shortref, parent):
-        dlg = cls.create(ref, shortref, parent)
-        if dlg is None:
-            return
-        parent.emit(SIGNAL(archive), ref, dlg.fmt, dlg.prefix, dlg.filename)
-        qtutils.information('File Saved', 'File saved to "%s"' % dlg.filename)
-
     def __init__(self, ref, shortref=None, parent=None):
         QtGui.QDialog.__init__(self, parent)
-        self.setWindowModality(QtCore.Qt.WindowModal)
+        self.setWindowModality(Qt.WindowModal)
 
         # input
         self.ref = ref
@@ -137,11 +129,15 @@ class GitArchiveDialog(QtGui.QDialog):
         self.connect(self.prefix_group, SIGNAL('expanded(bool)'),
                      self.prefix_group_expanded)
 
-        self.connect(self.browse, SIGNAL('clicked()'), self.choose_filename)
+        self.connect(self, SIGNAL('accepted()'), self.archive_saved)
 
-        self.connect(self.cancel, SIGNAL('clicked()'), self.reject)
-        self.connect(self.save, SIGNAL('clicked()'), self.save_archive)
+        qtutils.connect_button(self.browse, self.choose_filename)
+        qtutils.connect_button(self.cancel, self.reject)
+        qtutils.connect_button(self.save, self.save_archive)
 
+    def archive_saved(self):
+        cmds.do(cmds.Archive, self.ref, self.fmt, self.prefix, self.filename)
+        qtutils.information('File Saved', 'File saved to "%s"' % self.filename)
 
     def save_archive(self):
         filename = self.filename
@@ -186,7 +182,7 @@ class GitArchiveDialog(QtGui.QDialog):
         text = self.strip_exts(unicode(self.filetext.text()))
         self.filename = '%s.%s' % (text, self.fmt)
         self.filetext.setText(self.filename)
-        self.filetext.setFocus(True)
+        self.filetext.setFocus()
         if '/' in text:
             start = text.rindex('/') + 1
         else:
@@ -195,9 +191,9 @@ class GitArchiveDialog(QtGui.QDialog):
 
     def prefix_group_expanded(self, expanded):
         if expanded:
-            self.prefix_text.setFocus(True)
+            self.prefix_text.setFocus()
         else:
-            self.filetext.setFocus(True)
+            self.filetext.setFocus()
 
 
 if __name__ == '__main__':
