@@ -1,26 +1,31 @@
 from PyQt4 import QtGui
 from PyQt4 import QtCore
+from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
 import cola
+from cola import cmds
 from cola import gitcmds
-from cola import signals
+from cola import utils
+from cola.cmds import Interaction
 from cola.qt import create_toolbutton
+from cola.qtutils import add_action
 from cola.qtutils import confirm
-from cola.qtutils import connect_action
 from cola.qtutils import connect_action_bool
 from cola.qtutils import connect_button
-from cola.qtutils import emit
-from cola.qtutils import log
-from cola.qtutils import relay_signal
 from cola.qtutils import options_icon
 from cola.qtutils import save_icon
 from cola.qtutils import tr
 from cola.widgets import defs
 from cola.prefs import diff_font
+from cola.prefs import tabwidth
+from cola.prefs import textwidth
+from cola.prefs import linebreak
 from cola.dag.model import DAG
 from cola.dag.model import RepoReader
 from cola.widgets.selectcommits import select_commits
+from cola.widgets.text import HintedLineEdit
+from cola.widgets.text import HintedTextEdit
 
 
 class CommitMessageEditor(QtGui.QWidget):
@@ -29,24 +34,34 @@ class CommitMessageEditor(QtGui.QWidget):
 
         self.model = model
         self.notifying = False
-        self.summary_placeholder = u'Commit summary'
-        self.description_placeholder = u'Extended description...'
 
-        # Palette for normal text
-        self.default_palette = QtGui.QPalette(self.palette())
+        self._linebreak = None
+        self._textwidth = None
+        self._tabwidth = None
 
-        # Palette used for the placeholder text
-        self.placeholder_palette = pal = QtGui.QPalette(self.palette())
-        color = pal.text().color()
-        color.setAlpha(128)
-        pal.setColor(QtGui.QPalette.Active, QtGui.QPalette.Text, color)
-        pal.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.Text, color)
+        # Actions
+        self.signoff_action = add_action(self, cmds.SignOff.NAME,
+                                         cmds.run(cmds.SignOff),
+                                         cmds.SignOff.SHORTCUT)
+        self.signoff_action.setToolTip('Sign off on this commit')
+
+        self.commit_action = add_action(self, 'Commit@@verb',
+                                        self.commit,
+                                        'Ctrl+Return')
+        self.commit_action.setToolTip(tr('Commit staged changes'))
+
+        # Widgets
+        self.summary = CommitSummaryLineEdit()
+        self.summary.extra_actions.append(self.signoff_action)
+        self.summary.extra_actions.append(self.commit_action)
 
-        self.summary = QtGui.QLineEdit()
         self.description = CommitMessageTextEdit()
+        self.description.extra_actions.append(self.signoff_action)
+        self.description.extra_actions.append(self.commit_action)
 
+        commit_button_tooltip = 'Commit staged changes\nShortcut: Ctrl+Enter'
         self.commit_button = create_toolbutton(text='Commit@@verb',
-                                               tooltip='Commit staged changes',
+                                               tooltip=commit_button_tooltip,
                                                icon=save_icon())
 
         self.actions_menu = QtGui.QMenu()
@@ -55,21 +70,23 @@ class CommitMessageEditor(QtGui.QWidget):
         self.actions_button.setMenu(self.actions_menu)
         self.actions_button.setPopupMode(QtGui.QToolButton.InstantPopup)
 
-        # Amend checkbox
-        self.signoff_action = self.actions_menu.addAction(tr('Sign Off'))
-        self.signoff_action.setToolTip('Sign off on this commit')
-        self.signoff_action.setShortcut('Ctrl+i')
-
-        self.commit_action = self.actions_menu.addAction(tr('Commit@@verb'))
-        self.commit_action.setToolTip(tr('Commit staged changes'))
-        self.commit_action.setShortcut('Ctrl+m')
-
+        self.actions_menu.addAction(self.signoff_action)
+        self.actions_menu.addAction(self.commit_action)
         self.actions_menu.addSeparator()
+
+        # Amend checkbox
         self.amend_action = self.actions_menu.addAction(tr('Amend Last Commit'))
         self.amend_action.setCheckable(True)
 
+        # Line wrapping
+        self.actions_menu.addSeparator()
+        self.autowrap_action = self.actions_menu.addAction(
+                tr('Auto-Wrap Lines'))
+        self.autowrap_action.setCheckable(True)
+        self.autowrap_action.setChecked(linebreak())
+
         self.prev_commits_menu = self.actions_menu.addMenu(
-                    'Load Previous Commit Message')
+                tr('Load Previous Commit Message'))
         self.connect(self.prev_commits_menu, SIGNAL('aboutToShow()'),
                      self.build_prev_commits_menu)
 
@@ -87,26 +104,32 @@ class CommitMessageEditor(QtGui.QWidget):
         self.mainlayout.addWidget(self.description)
         self.setLayout(self.mainlayout)
 
-        relay_signal(self, self.description,
-                     SIGNAL(signals.load_previous_message))
-
         connect_button(self.commit_button, self.commit)
-        connect_action(self.commit_action, self.commit)
-        connect_action(self.signoff_action, emit(self, signals.signoff))
 
-        cola.notifier().connect(signals.amend, self.amend_action.setChecked)
+        notifier = cola.notifier()
+        notifier.connect(notifier.AMEND, self.amend_action.setChecked)
 
         # Broadcast the amend mode
-        connect_action_bool(self.amend_action, emit(self, signals.amend_mode))
+        connect_action_bool(self.amend_action, cmds.run(cmds.AmendMode))
 
-        self.model.add_message_observer(self.model.message_commit_message_changed,
-                                        self.set_commit_message)
+        # Handle the one-off autowrapping
+        connect_action_bool(self.autowrap_action, self.set_linebreak)
+
+        self.model.add_observer(self.model.message_commit_message_changed,
+                                self.set_commit_message)
 
         self.connect(self.summary, SIGNAL('returnPressed()'),
-                     self.summary_return_pressed)
+                     self.focus_description)
 
-        self.connect(self.summary, SIGNAL('cursorPositionChanged(int,int)'),
-                     lambda x, y: self.emit_summary_position())
+        self.connect(self.summary, SIGNAL('downPressed()'),
+                     self.focus_description)
+
+        self.connect(self.summary, SIGNAL('cursorPosition(int,int)'),
+                     self.emit_position)
+
+        self.connect(self.description, SIGNAL('cursorPosition(int,int)'),
+                     # description starts at line 2
+                     lambda row, col: self.emit_position(row + 2, col))
 
         # Keep model informed of changes
         self.connect(self.summary, SIGNAL('textChanged(QString)'),
@@ -115,100 +138,46 @@ class CommitMessageEditor(QtGui.QWidget):
         self.connect(self.description, SIGNAL('textChanged()'),
                      self.commit_message_changed)
 
-        self.connect(self.description, SIGNAL('cursorPositionChanged()'),
-                     self.emit_cursor_position)
+        self.connect(self.description, SIGNAL('shiftTab()'),
+                     self.focus_summary)
 
-        self.setFocusProxy(self.summary)
         self.setFont(diff_font())
 
-        self.summary.installEventFilter(self)
-        self.description.installEventFilter(self)
-
-        self.enable_placeholder_summary(True)
-        self.enable_placeholder_description(True)
+        self.summary.enable_hint(True)
+        self.description.enable_hint(True)
 
         self.commit_button.setEnabled(False)
         self.commit_action.setEnabled(False)
 
-    def is_summary_placeholder(self):
-        summary = unicode(self.summary.text()).strip()
-        return summary == self.summary_placeholder
+        self.setFocusProxy(self.summary)
 
-    def is_description_placeholder(self):
-        description = unicode(self.description.toPlainText()).strip()
-        return description == self.description_placeholder
+        self.set_tabwidth(tabwidth())
+        self.set_textwidth(textwidth())
+        self.set_linebreak(linebreak())
 
-    def eventFilter(self, obj, event):
-        if obj == self.summary:
-            if event.type() == QtCore.QEvent.FocusIn:
-                self.emit_summary_position()
-                if self.is_summary_placeholder():
-                    self.enable_placeholder_summary(False)
+        # Allow tab to jump from the summary to the description
+        self.setTabOrder(self.summary, self.description)
 
-            elif event.type() == QtCore.QEvent.FocusOut:
-                if not bool(self.commit_summary()):
-                    self.enable_placeholder_summary(True)
+    def set_initial_size(self):
+        self.setMaximumHeight(133)
+        QtCore.QTimer.singleShot(1, self.restore_size)
 
-        elif obj == self.description:
-            if event.type() == QtCore.QEvent.FocusIn:
-                if self.is_description_placeholder():
-                    self.enable_placeholder_description(False)
+    def restore_size(self):
+        self.setMaximumHeight(2 ** 13)
 
-            elif event.type() == QtCore.QEvent.FocusOut:
-                if not bool(self.commit_description()):
-                    self.enable_placeholder_description(True)
+    def focus_summary(self):
+        self.summary.setFocus()
 
-        return False
+    def focus_description(self):
+        self.description.setFocus()
 
-    def enable_placeholder_summary(self, placeholder):
-        blocksignals = self.summary.blockSignals(True)
-        if placeholder:
-            self.summary.setText(self.summary_placeholder)
-        else:
-            self.summary.clear()
-        self.summary.setCursorPosition(0)
-        self.summary.blockSignals(blocksignals)
-        self.set_placeholder_palette(self.summary, placeholder)
-
-    def enable_placeholder_description(self, placeholder):
-        blocksignals = self.description.blockSignals(True)
-        if placeholder:
-            self.description.setPlainText(self.description_placeholder)
-        else:
-            self.description.clear()
-        self.description.blockSignals(blocksignals)
-        self.set_placeholder_palette(self.description, placeholder)
-
-    def set_placeholder_palette(self, widget, placeholder):
-        if placeholder:
-            widget.setPalette(self.placeholder_palette)
-        else:
-            widget.setPalette(self.default_palette)
-
-    def summary_return_pressed(self):
-        if bool(self.commit_summary()):
-            self.description.setFocus(True)
-
-    def commit_summary(self):
-        """Return the commit summary as unicode"""
-        summary = unicode(self.summary.text()).strip()
-        if summary != self.summary_placeholder:
-            return summary
-        else:
-            return u''
-
-    def commit_description(self):
-        """Return the commit description as unicode"""
-        description = unicode(self.description.toPlainText()).strip()
-        if description != self.description_placeholder:
-            return description
-        else:
-            return u''
-
-    def commit_message(self):
+    def commit_message(self, raw=True):
         """Return the commit message as a unicode string"""
-        summary = self.commit_summary()
-        description = self.commit_description()
+        summary = self.summary.value()
+        if raw:
+            description = self.description.value()
+        else:
+            description = self.formatted_description()
         if summary and description:
             return summary + u'\n\n' + description
         elif summary:
@@ -218,26 +187,30 @@ class CommitMessageEditor(QtGui.QWidget):
         else:
             return u''
 
+    def formatted_description(self):
+        text = self.description.value()
+        if not self._linebreak:
+            return text
+        return utils.word_wrap(text, self._tabwidth, self._textwidth)
+
     def commit_message_changed(self, value=None):
         """Update the model when values change"""
         self.notifying = True
         message = self.commit_message()
         self.model.set_commitmsg(message)
-        self.update_placeholder_state()
+        self.refresh_palettes()
         self.notifying = False
         self.update_actions()
 
     def update_actions(self):
-        commit_enabled = bool(self.commit_summary())
+        commit_enabled = bool(self.summary.value())
         self.commit_button.setEnabled(commit_enabled)
         self.commit_action.setEnabled(commit_enabled)
 
-    def update_placeholder_state(self):
-        """Update the color palette for the placeholder text"""
-        self.set_placeholder_palette(self.summary,
-                                     self.is_summary_placeholder())
-        self.set_placeholder_palette(self.description,
-                                     self.is_description_placeholder())
+    def refresh_palettes(self):
+        """Update the color palette for the hint text"""
+        self.summary.refresh_palette()
+        self.description.refresh_palette()
 
     def set_commit_message(self, message):
         """Set the commit message to match the observed model"""
@@ -282,7 +255,7 @@ class CommitMessageEditor(QtGui.QWidget):
 
         # Update summary
         if not summary and not self.summary.hasFocus():
-            summary = self.summary_placeholder
+            summary = self.summary.hint()
 
         blocksignals = self.summary.blockSignals(True)
         self.summary.setText(summary)
@@ -291,27 +264,42 @@ class CommitMessageEditor(QtGui.QWidget):
 
         # Update description
         if not description and not self.description.hasFocus():
-            description = self.description_placeholder
+            description = self.description.hint()
 
         blocksignals = self.description.blockSignals(True)
         self.description.setPlainText(description)
         self.description.blockSignals(blocksignals)
 
         # Update text color
-        self.update_placeholder_state()
+        self.refresh_palettes()
 
         # Focus the empty summary or description
         if focus_summary:
-            self.summary.setFocus(True)
-            self.emit_summary_position()
+            self.summary.setFocus()
+            self.summary.emit_position()
         elif focus_description:
-            self.description.setFocus(True)
-            self.emit_cursor_position()
+            self.description.setFocus()
+            self.description.emit_position()
         else:
-            self.emit_summary_position()
+            self.summary.emit_position()
 
         self.update_actions()
 
+    def set_tabwidth(self, width):
+        self._tabwidth = width
+        self.description.set_tabwidth(width)
+
+    def set_textwidth(self, width):
+        self._textwidth = width
+        self.description.set_textwidth(width)
+
+    def set_linebreak(self, brk):
+        self._linebreak = brk
+        self.description.set_linebreak(brk)
+        blocksignals = self.autowrap_action.blockSignals(True)
+        self.autowrap_action.setChecked(brk)
+        self.autowrap_action.blockSignals(blocksignals)
+
     def setFont(self, font):
         """Pass the setFont() calls down to the text widgets"""
         self.summary.setFont(font)
@@ -323,22 +311,12 @@ class CommitMessageEditor(QtGui.QWidget):
         self.amend_action.setChecked(checked)
         self.amend_action.blockSignals(blocksignals)
 
-    def emit_summary_position(self):
-        cols = self.summary.cursorPosition()
-        self.emit(SIGNAL('cursorPosition(int,int)'), 1, cols)
-
-    def emit_cursor_position(self):
-        """Update the UI with the current row and column."""
-        cursor = self.description.textCursor()
-        position = cursor.position()
-        txt = unicode(self.description.toPlainText())
-        rows = txt[:position].count('\n') + 1 + 2 # description starts at 2
-        cols = cursor.columnNumber()
-        self.emit(SIGNAL('cursorPosition(int,int)'), rows, cols)
+    def emit_position(self, row, col):
+        self.emit(SIGNAL('cursorPosition(int,int)'), row, col)
 
     def commit(self):
         """Attempt to create a commit from the index and commit message."""
-        if not bool(self.commit_summary()):
+        if not bool(self.summary.value()):
             # Describe a good commit message
             error_msg = tr(''
                 'Please supply a commit message.\n\n'
@@ -346,13 +324,11 @@ class CommitMessageEditor(QtGui.QWidget):
                 '- First line: Describe in one sentence what you did.\n'
                 '- Second line: Blank\n'
                 '- Remaining lines: Describe why this change is good.\n')
-            log(1, error_msg)
-            cola.notifier().broadcast(signals.information,
-                                      'Missing Commit Message',
-                                      error_msg)
+            Interaction.log(error_msg)
+            Interaction.information('Missing Commit Message', error_msg)
             return
 
-        msg = self.commit_message()
+        msg = self.commit_message(raw=False)
 
         if not self.model.staged:
             error_msg = tr(''
@@ -369,11 +345,9 @@ class CommitMessageEditor(QtGui.QWidget):
                                icon=save_icon()):
                     return
             else:
-                cola.notifier().broadcast(signals.information,
-                                          'Nothing to commit',
-                                          error_msg)
+                Interaction.information('Nothing to commit', error_msg)
                 return
-            cola.notifier().broadcast(signals.stage_modified)
+            cmds.do(cmds.StageModified)
 
         # Warn that amending published commits is generally bad
         amend = self.amend_action.isChecked()
@@ -386,8 +360,7 @@ class CommitMessageEditor(QtGui.QWidget):
                         'Amend Commit',
                         default=False, icon=save_icon())):
             return
-        # Perform the commit
-        cola.notifier().broadcast(signals.commit, amend, msg)
+        cmds.do(cmds.Commit, amend, msg)
 
     def build_prev_commits_menu(self):
         dag = DAG('HEAD', 6)
@@ -403,7 +376,7 @@ class CommitMessageEditor(QtGui.QWidget):
         menu.clear()
         for c in menu_commits:
             menu.addAction(c.summary,
-                           lambda c=c: self.load_previous_message(c.sha1))
+                           cmds.run(cmds.LoadPreviousMessage, c.sha1))
 
         if len(commits) == 6:
             menu.addSeparator()
@@ -416,24 +389,108 @@ class CommitMessageEditor(QtGui.QWidget):
         if not sha1s:
             return
         sha1 = sha1s[0]
-        self.load_previous_message(sha1)
-
-    def load_previous_message(self, sha1):
-        self.emit(SIGNAL(signals.load_previous_message), sha1)
+        cmds.do(cmds.LoadPreviousMessage, sha1)
 
 
-class CommitMessageTextEdit(QtGui.QTextEdit):
+class CommitSummaryLineEdit(HintedLineEdit):
     def __init__(self, parent=None):
-        QtGui.QTextEdit.__init__(self, parent)
+        hint = u'Commit summary'
+        HintedLineEdit.__init__(self, hint, parent)
+        self.extra_actions = []
 
-        policy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum,
-                                   QtGui.QSizePolicy.Minimum)
-        self.setSizePolicy(policy)
-        self.setLineWrapMode(QtGui.QTextEdit.NoWrap)
-        self.setAcceptRichText(False)
+    def keyPressEvent(self, event):
+        if event.key() == Qt.Key_Down:
+            position = self.cursorPosition()
+            curtext = unicode(self.text())
+            if position == len(curtext):
+                self.emit(SIGNAL('downPressed()'))
+                event.accept()
+                return
+        if event.key() in (Qt.Key_Return, Qt.Key_Enter):
+            self.emit(SIGNAL('returnPressed()'))
+            event.accept()
+            return
+        HintedLineEdit.keyPressEvent(self, event)
 
-    def setFont(self, font):
-        QtGui.QTextEdit.setFont(self, font)
-        height = QtGui.QFontMetrics(font).height() * 2
-        height += defs.spacing * 3
-        self.setMinimumSize(QtCore.QSize(1, height))
+    def contextMenuEvent(self, event):
+        menu = self.createStandardContextMenu()
+        if self.extra_actions:
+            menu.addSeparator()
+        for action in self.extra_actions:
+            menu.addAction(action)
+        menu.exec_(self.mapToGlobal(event.pos()))
+
+
+class CommitMessageTextEdit(HintedTextEdit):
+    def __init__(self, parent=None):
+        hint = u'Extended description...'
+        HintedTextEdit.__init__(self, hint, parent)
+        self.extra_actions = []
+        self.setMinimumSize(QtCore.QSize(1, 1))
+
+        self.action_emit_shift_tab = add_action(self,
+                'Shift Tab', self.emit_shift_tab, 'Shift+tab')
+
+        self.installEventFilter(self)
+
+    def eventFilter(self, obj, event):
+        if event.type() == QtCore.QEvent.FocusIn:
+            height = QtGui.QFontMetrics(self.font()).height() * 3
+            height += defs.spacing * 4
+            self.setMinimumSize(QtCore.QSize(1, height))
+
+        elif event.type() == QtCore.QEvent.FocusOut:
+            self.setMinimumSize(QtCore.QSize(1, 1))
+
+        return False
+
+    def contextMenuEvent(self, event):
+        menu = self.createStandardContextMenu()
+        if self.extra_actions:
+            menu.addSeparator()
+        for action in self.extra_actions:
+            menu.addAction(action)
+        menu.exec_(self.mapToGlobal(event.pos()))
+
+    def keyPressEvent(self, event):
+        if event.key() == Qt.Key_Up:
+            cursor = self.textCursor()
+            position = cursor.position()
+            if position == 0:
+                if cursor.hasSelection():
+                    cursor.setPosition(0)
+                    self.setTextCursor(cursor)
+                else:
+                    self.emit_shift_tab()
+                event.accept()
+                return
+            text_before = unicode(self.toPlainText())[:position]
+            lines_before = text_before.count('\n')
+            if lines_before == 0:
+                if event.modifiers() & Qt.ShiftModifier:
+                    mode = QtGui.QTextCursor.KeepAnchor
+                else:
+                    mode = QtGui.QTextCursor.MoveAnchor
+                cursor.setPosition(0, mode)
+                self.setTextCursor(cursor)
+                event.accept()
+                return
+        elif event.key() == Qt.Key_Down:
+            cursor = self.textCursor()
+            position = cursor.position()
+            all_text = unicode(self.toPlainText())
+            text_after = all_text[position:]
+            lines_after = text_after.count('\n')
+            if lines_after == 0:
+                if event.modifiers() & Qt.ShiftModifier:
+                    mode = QtGui.QTextCursor.KeepAnchor
+                else:
+                    mode = QtGui.QTextCursor.MoveAnchor
+                cursor.setPosition(len(all_text), mode)
+                self.setTextCursor(cursor)
+                event.accept()
+                return
+        HintedTextEdit.keyPressEvent(self, event)
+
+    def emit_shift_tab(self):
+        self.emit(SIGNAL('shiftTab()'))
