@@ -1,56 +1,45 @@
 import os
 
 from PyQt4 import QtGui
-from PyQt4 import QtCore
 from PyQt4.QtCore import Qt, SIGNAL
 
 import cola
-from cola import guicmds
+from cola import cmds
 from cola import qtutils
-from cola import signals
-from cola.prefs import diff_font
-from cola.prefs import tab_width
-from cola.qt import DiffSyntaxHighlighter
-from cola.qtutils import SLOT
+from cola.cmds import run
+from cola.widgets import defs
+from cola.widgets.text import DiffTextEdit
 
 
-class DiffView(QtGui.QTextEdit):
+class DiffEditor(DiffTextEdit):
     def __init__(self, parent):
-        QtGui.QTextEdit.__init__(self, parent)
-        self.setMinimumSize(QtCore.QSize(1, 1))
-        self.setLineWrapMode(QtGui.QTextEdit.NoWrap)
-        self.setAcceptRichText(False)
-        self.setCursorWidth(2)
-        self.setTextInteractionFlags(Qt.TextSelectableByKeyboard |
-                                     Qt.TextSelectableByMouse)
-        # Diff/patch syntax highlighter
-        self.syntax = DiffSyntaxHighlighter(self.document())
-        self.setFont(diff_font())
-        self.set_tab_width(tab_width())
-
-    def set_tab_width(self, tab_width):
-        display_font = self.font()
-        space_width = QtGui.QFontMetrics(display_font).width(' ')
-        self.setTabStopWidth(tab_width * space_width)
-
-
-class DiffTextEdit(DiffView):
-    def __init__(self, parent):
-        DiffView.__init__(self, parent)
+        DiffTextEdit.__init__(self, parent)
         self.model = model = cola.model()
+        self.mode = self.model.mode_none
 
-        # Install diff shortcut keys for stage/unstage
         self.action_process_section = qtutils.add_action(self,
                 'Process Section',
-                self.apply_section, QtCore.Qt.Key_H)
+                self.apply_section, Qt.Key_H)
         self.action_process_selection = qtutils.add_action(self,
                 'Process Selection',
-                self.apply_selection, QtCore.Qt.Key_S)
-        # Context menu actions
+                self.apply_selection, Qt.Key_S)
+
+        self.launch_editor = qtutils.add_action(self,
+                cmds.LaunchEditor.NAME, run(cmds.LaunchEditor),
+                cmds.LaunchEditor.SHORTCUT,
+                'Return', 'Enter')
+        self.launch_editor.setIcon(qtutils.options_icon())
+
+        self.launch_difftool = qtutils.add_action(self,
+                cmds.LaunchDifftool.NAME, run(cmds.LaunchDifftool),
+                cmds.LaunchDifftool.SHORTCUT)
+        self.launch_difftool.setIcon(qtutils.icon('git.svg'))
+
         self.action_stage_selection = qtutils.add_action(self,
                 self.tr('Stage &Selected Lines'),
                 self.stage_selection)
         self.action_stage_selection.setIcon(qtutils.icon('add.svg'))
+        self.action_stage_selection.setShortcut(Qt.Key_S)
 
         self.action_revert_selection = qtutils.add_action(self,
                 self.tr('Revert Selected Lines...'),
@@ -61,14 +50,16 @@ class DiffTextEdit(DiffView):
                 self.tr('Unstage &Selected Lines'),
                 self.unstage_selection)
         self.action_unstage_selection.setIcon(qtutils.icon('remove.svg'))
+        self.action_unstage_selection.setShortcut(Qt.Key_S)
 
         self.action_apply_selection = qtutils.add_action(self,
                 self.tr('Apply Diff Selection to Work Tree'),
                 self.stage_selection)
         self.action_apply_selection.setIcon(qtutils.apply_icon())
 
-        model.add_message_observer(model.message_diff_text_changed,
-                                   self.setPlainText)
+        model.add_observer(model.message_mode_about_to_change,
+                           self._mode_about_to_change)
+        model.add_observer(model.message_diff_text_changed, self.setPlainText)
 
         self.connect(self, SIGNAL('copyAvailable(bool)'),
                      self.enable_selection_actions)
@@ -79,19 +70,21 @@ class DiffTextEdit(DiffView):
         menu = QtGui.QMenu(self)
         s = cola.selection()
 
-        if self.mode == self.model.mode_worktree:
+        if self.model.stageable():
             if s.modified and s.modified[0] in cola.model().submodules:
-                menu.addAction(qtutils.icon('add.svg'),
-                               self.tr('Stage'),
-                               SLOT(signals.stage, s.modified))
+                action = menu.addAction(qtutils.icon('add.svg'),
+                                        self.tr(cmds.Stage.NAME),
+                                        cmds.run(cmds.Stage, s.modified))
+                action.setShortcut(cmds.Stage.SHORTCUT)
                 menu.addAction(qtutils.git_icon(),
                                self.tr('Launch git-cola'),
-                               SLOT(signals.open_repo,
+                               cmds.run(cmds.OpenRepo,
                                     os.path.abspath(s.modified[0])))
             elif s.modified:
-                menu.addAction(qtutils.icon('add.svg'),
-                               self.tr('Stage Section'),
-                               self.stage_section)
+                action = menu.addAction(qtutils.icon('add.svg'),
+                                        self.tr('Stage Section'),
+                                        self.stage_section)
+                action.setShortcut(Qt.Key_H)
                 menu.addAction(self.action_stage_selection)
                 menu.addSeparator()
                 menu.addAction(qtutils.icon('undo.svg'),
@@ -99,67 +92,71 @@ class DiffTextEdit(DiffView):
                                self.revert_section)
                 menu.addAction(self.action_revert_selection)
 
-        elif self.mode == self.model.mode_index:
+        if self.model.unstageable():
             if s.staged and s.staged[0] in cola.model().submodules:
-                menu.addAction(qtutils.icon('remove.svg'),
-                               self.tr('Unstage'),
-                               SLOT(signals.unstage, s.staged))
+                action = menu.addAction(qtutils.icon('remove.svg'),
+                                        self.tr(cmds.Unstage.NAME),
+                                        cmds.do(cmds.Unstage, s.staged))
+                action.setShortcut(cmds.Unstage.SHORTCUT)
                 menu.addAction(qtutils.git_icon(),
                                self.tr('Launch git-cola'),
-                               SLOT(signals.open_repo,
+                               cmds.do(cmds.OpenRepo,
                                     os.path.abspath(s.staged[0])))
-            else:
-                menu.addAction(qtutils.icon('remove.svg'),
-                               self.tr('Unstage Section'),
-                               self.unstage_section)
+            elif s.staged:
+                action = menu.addAction(qtutils.icon('remove.svg'),
+                                        self.tr('Unstage Section'),
+                                        self.unstage_section)
+                action.setShortcut(Qt.Key_H)
                 menu.addAction(self.action_unstage_selection)
 
-        elif self.mode == self.model.mode_grep:
-            menu.addAction(qtutils.icon('open.svg'),
-                           self.tr('Launch Editor'),
-                           lambda: guicmds.goto_grep(self.selected_line()))
+        if self.model.stageable() or self.model.unstageable():
+            menu.addSeparator()
+            menu.addAction(self.launch_editor)
+            menu.addAction(self.launch_difftool)
 
         menu.addSeparator()
-        menu.addAction(qtutils.icon('edit-copy.svg'),
-                       'Copy', self.copy)
-        menu.addAction(qtutils.icon('edit-select-all.svg'),
-                       'Select All', self.selectAll)
+        action = menu.addAction(qtutils.icon('edit-copy.svg'),
+                                'Copy', self.copy)
+        action.setShortcut(QtGui.QKeySequence.Copy)
+
+        action = menu.addAction(qtutils.icon('edit-select-all.svg'),
+                                'Select All', self.selectAll)
+        action.setShortcut(QtGui.QKeySequence.SelectAll)
         menu.exec_(self.mapToGlobal(event.pos()))
 
+    def wheelEvent(self, event):
+        if event.modifiers() & Qt.ControlModifier:
+            # Intercept the Control modifier to not resize the text
+            # when doing control+mousewheel
+            event.accept()
+            event = QtGui.QWheelEvent(event.pos(), event.delta(),
+                                      Qt.NoButton,
+                                      Qt.NoModifier,
+                                      event.orientation())
+        return DiffTextEdit.wheelEvent(self, event)
+
+    def _mode_about_to_change(self, mode):
+        self.mode = mode
+
     def setPlainText(self, text):
         """setPlainText(str) while retaining scrollbar positions"""
+        highlight = (self.mode != self.model.mode_none and
+                     self.mode != self.model.mode_untracked)
+        self.highlighter.set_enabled(highlight)
         scrollbar = self.verticalScrollBar()
         if scrollbar:
             scrollvalue = scrollbar.value()
         if text is not None:
-            QtGui.QTextEdit.setPlainText(self, text)
+            DiffTextEdit.setPlainText(self, text)
             if scrollbar:
                 scrollbar.setValue(scrollvalue)
 
-    # Accessors
-    mode = property(lambda self: self.model.mode)
-
     def offset_and_selection(self):
         cursor = self.textCursor()
         offset = cursor.position()
         selection = unicode(cursor.selection().toPlainText())
         return offset, selection
 
-    def selected_line(self):
-        cursor = self.textCursor()
-        offset = cursor.position()
-        contents = unicode(self.toPlainText())
-        while (offset >= 1
-                and contents[offset-1]
-                and contents[offset-1] != '\n'):
-            offset -= 1
-        data = contents[offset:]
-        if '\n' in data:
-            line, rest = data.split('\n', 1)
-        else:
-            line = data
-        return line
-
     # Mutators
     def enable_selection_actions(self, enabled):
         self.action_apply_selection.setEnabled(enabled)
@@ -169,16 +166,16 @@ class DiffTextEdit(DiffView):
 
     def apply_section(self):
         s = cola.single_selection()
-        if self.mode == self.model.mode_worktree and s.modified:
+        if self.model.stageable() and s.modified:
             self.stage_section()
-        elif self.mode == self.model.mode_index:
+        elif self.model.unstageable():
             self.unstage_section()
 
     def apply_selection(self):
         s = cola.single_selection()
-        if self.mode == self.model.mode_worktree and s.modified:
+        if self.model.stageable() and s.modified:
             self.stage_selection()
-        elif self.mode == self.model.mode_index:
+        elif self.model.unstageable():
             self.unstage_selection()
 
     def stage_section(self):
@@ -204,7 +201,7 @@ class DiffTextEdit(DiffView):
                                'These changes cannot be recovered.',
                                'Revert the uncommitted changes?',
                                'Revert Section',
-                               default=False,
+                               default=True,
                                icon=qtutils.icon('undo.svg')):
             return
         self.process_diff_selection(staged=False, apply_to_worktree=True,
@@ -217,7 +214,7 @@ class DiffTextEdit(DiffView):
                                'These changes cannot be recovered.',
                                'Revert the uncommitted changes?',
                                'Revert Selected Lines',
-                               default=False,
+                               default=True,
                                icon=qtutils.icon('undo.svg')):
             return
         self.process_diff_selection(staged=False, apply_to_worktree=True,
@@ -228,9 +225,5 @@ class DiffTextEdit(DiffView):
                                reverse=False):
         """Implement un/staging of selected lines or sections."""
         offset, selection = self.offset_and_selection()
-        cola.notifier().broadcast(signals.apply_diff_selection,
-                                  staged,
-                                  selected,
-                                  offset,
-                                  selection,
-                                  apply_to_worktree)
+        cmds.do(cmds.ApplyDiffSelection,
+                staged, selected, offset, selection, apply_to_worktree)
